@@ -12,11 +12,10 @@ import sklearn.manifold as man
 import sklearn.utils.sparsefuncs as sf
 from packaging import version
 import warnings
-from numba.core.errors import NumbaPerformanceWarning
+from numba.core.errors import NumbaWarning
+warnings.filterwarnings("ignore", category=NumbaWarning)
 
-warnings.filterwarnings("ignore", category=NumbaPerformanceWarning)
-
-__version__ = "0.7.6"
+__version__ = "0.7.7"
 
 """
 Copyright 2018, Alexander J. Tarashansky, All rights reserved.
@@ -121,11 +120,11 @@ class SAM(object):
         self,
         div=1,
         downsample=0,
-        sum_norm=None,
+        sum_norm="cell_median",
         norm="log",
         min_expression=1,
-        thresh_low=0.01,
-        thresh_high=0.99,
+        thresh_low=0.0,
+        thresh_high=0.96,
         thresh = None,
         filter_genes=True,
     ):
@@ -142,7 +141,7 @@ class SAM(object):
             The factor by which to randomly downsample the data. If 0, the
             data will not be downsampled.
 
-        sum_norm : str or float, optional, default None
+        sum_norm : str or float, optional, default "cell_median"
             If a float, the total number of transcripts in each cell will be
             normalized to this value prior to normalization and filtering.
             Otherwise, nothing happens. If 'cell_median', each cell is
@@ -162,12 +161,12 @@ class SAM(object):
             expressed. Gene expression values less than 'min_expression' are
             set to zero.
 
-        thresh_low : float, optional, default 0.01
+        thresh_low : float, optional, default 0.0
             Keep genes expressed in greater than 'thresh_low'*100 % of cells,
             where a gene is considered expressed if its expression value
             exceeds 'min_expression'.
 
-        thresh_high : float, optional, default 0.99
+        thresh_high : float, optional, default 0.96
             Keep genes expressed in less than 'thresh_high'*100 % of cells,
             where a gene is considered expressed if its expression value
             exceeds 'min_expression'.
@@ -439,7 +438,8 @@ class SAM(object):
             if "X_disp" not in list(self.adata.layers.keys()):
                 self.adata.layers["X_disp"] = self.adata.X
             save_sparse_file = None
-
+        self.adata.uns['path_to_file'] = filename
+        self.adata_raw.uns['path_to_file'] = filename        
         if save_sparse_file is not None:
             if save_sparse_file.split(".")[-1] == "p":
                 self.save_sparse_data(save_sparse_file)
@@ -796,79 +796,6 @@ class SAM(object):
         weights = ((dispersions / dispersions.max()) ** 0.5).flatten()
         return weights
 
-    def calculate_regression_PCs(self, genes=None, npcs=None):
-        """Computes the contribution of the gene IDs in 'genes' to each
-        principal component (PC) of the filtered expression data as the mean of
-        the absolute value of the corresponding gene loadings. High values
-        correspond to PCs that are highly correlated with the features in
-        'genes'. These PCs can then be regressed out of the data using
-        'regress_genes'.
-
-
-        Parameters
-        ----------
-        genes - numpy.array or list, default None
-            Genes for which contribution to each PC will be calculated. Set to None
-            if you know ahead of time which PC you wish to remove from the data using
-            'regress_genes'.
-
-        npcs - int, optional, default None
-            How many PCs to calculate when computing PCA of the filtered and
-            log-transformed expression data. If None, calculate all PCs.
-
-        Returns:
-        -------
-        x - numpy.array
-            Scores reflecting how correlated each PC is with the genes of
-            interest (ordered by decreasing eigenvalues).
-
-        """
-        from sklearn.decomposition import PCA
-
-        if npcs is None:
-            npcs = self.adata.X.shape[0]
-
-        pca = PCA(n_components=npcs)
-        pc = pca.fit_transform(self.adata.X.toarray())
-
-        self.regression_pca = pca
-        self.regression_pcs = pc
-
-        gene_names = np.array(list(self.adata.var_names))
-        if genes is not None:
-            idx = np.where(np.in1d(gene_names, genes))[0]
-            sx = pca.components_[:, idx]
-            x = np.abs(sx).mean(1)
-            return x
-        else:
-            return
-
-    def regress_genes(self, PCs):
-        """Regress out the principal components in 'PCs' from the filtered
-        expression data ('SAM.D'). Assumes 'calculate_regression_PCs' has
-        been previously called.
-
-        Parameters
-        ----------
-        PCs - int, numpy.array, list
-            The principal components to regress out of the expression data.
-
-        """
-
-        ind = [PCs]
-        ind = np.array(ind).flatten()
-        try:
-            y = self.adata.X.toarray() - self.regression_pcs[:, ind].dot(
-                self.regression_pca.components_[ind, :]
-                * self.adata.var["weights"].values
-            )
-        except BaseException:
-            y = self.adata.X.toarray() - self.regression_pcs[:, ind].dot(
-                self.regression_pca.components_[ind, :]
-            )
-
-        self.adata.X = sp.csr_matrix(y)
-
     def run(
         self,
         max_iter=10,
@@ -878,10 +805,10 @@ class SAM(object):
         num_norm_avg=50,
         k=20,
         distance="correlation",
-        preprocessing="Normalizer",
+        preprocessing="StandardScaler",
         npcs=None,
         n_genes=None,
-        weight_PCs=True,
+        weight_PCs=False,
         sparse_pca=False,
         proj_kwargs={},
         seed = 0,
@@ -914,7 +841,7 @@ class SAM(object):
             If 'tsne', generates a t-SNE embedding. If 'umap', generates a UMAP
             embedding. Otherwise, no embedding will be generated.
 
-        preprocessing - str, optional, default 'Normalizer'
+        preprocessing - str, optional, default 'StandardScaler'
             If 'Normalizer', use sklearn.preprocessing.Normalizer, which
             normalizes expression data prior to PCA such that each cell has
             unit L2 norm. If 'StandardScaler', use
@@ -935,7 +862,7 @@ class SAM(object):
             This is worth setting True for large datasets, where memory
             constraints start becoming noticeable.
 
-        weight_PCs - bool, optional, default True
+        weight_PCs - bool, optional, default False
             Scale the principal components by their eigenvalues. If many
             cell populations are expected, it is recommended to set this False
             as the populations may be found on lesser-varying PCs.
@@ -981,7 +908,7 @@ class SAM(object):
             "seed": seed,
             "components": components,
         }
-
+        self.adata.uns["run_args"] = self.run_args
         numcells = D.shape[0]
 
         if n_genes == None:
@@ -1034,11 +961,14 @@ class SAM(object):
 
             i += 1
             old = new
-
+            if i == 1:                
+                first=True
+            else:
+                first=False
             W = self.calculate_nnm(
                 n_genes=n_genes, preprocessing=preprocessing, npcs=npcs, num_norm_avg=nnas,
-                weight_PCs=weight_PCs, sparse_pca=sparse_pca,weight_mode=weight_mode,seed=seed,components=components
-            )
+                weight_PCs=weight_PCs, sparse_pca=sparse_pca,weight_mode=weight_mode,seed=seed,components=components,first=first
+                )                
             gc.collect()
             new = W
             err = ((new - old) ** 2).mean() ** 0.5
@@ -1063,17 +993,16 @@ class SAM(object):
                 print("Computing the diffusion UMAP embedding...")
             self.run_diff_umap(**proj_kwargs)
 
-        self.adata.uns["run_args"] = self.run_args
         elapsed = time.time() - tinit
         if verbose:
             print("Elapsed time: " + str(elapsed) + " seconds")
 
     def calculate_nnm(
         self, n_genes=3000, preprocessing='StandardScaler', npcs=150, num_norm_avg=50, weight_PCs=False, sparse_pca=False,
-        update_manifold=True,weight_mode='dispersion',seed=0,components=None
+        update_manifold=True,weight_mode='dispersion',seed=0,components=None,first=False
     ):
         if 'means' not in self.adata.var.keys() or 'variances' not in self.adata.var.keys():
-            print('Recomputing means and variances.')
+            print('Computing means and variances of genes.')
             mu,var = sf.mean_variance_axis(self.adata.X,axis=0)
             self.adata.var['means'] = mu
             self.adata.var['variances'] = var
@@ -1082,13 +1011,20 @@ class SAM(object):
         D = self.adata.X
         W = self.adata.var["weights"].values
 
-        k = self.run_args.get("k", 20)
-        distance = self.run_args.get("distance", "correlation")
+        k = self.adata.uns['run_args'].get("k", 20)
+        distance = self.adata.uns['run_args'].get("distance", "correlation")
 
         if n_genes is None:
             gkeep = np.arange(W.size)
         else:
-            gkeep = np.sort(np.argsort(-W)[:n_genes])
+            if first:
+                mu = np.array(list(self.adata.var['means']))
+                var = np.array(list(self.adata.var['variances']))
+                mu[mu==0]=1
+                dispersions = var/mu
+                gkeep = np.sort(np.argsort(-dispersions)[:n_genes])
+            else:    
+                gkeep = np.sort(np.argsort(-W)[:n_genes])
 
         if preprocessing == "Normalizer":
             Ds = D[:, gkeep]
@@ -1187,9 +1123,9 @@ class SAM(object):
                 edm.data[:] = 1-edm.data
                 edm = edm.tolil(); edm.setdiag(1); edm = edm.tocsr();
                 edm.data[edm.data<0]=0.001 #if negative correlation, set close to zero but not zero to preserve kNN structure
-                self.adata.uns['nnm'] = edm
+                self.adata.obsp['nnm'] = edm
             else:
-                self.adata.uns['nnm'] = EDM
+                self.adata.obsp['nnm'] = EDM
             W = self.dispersion_ranking_NN(EDM, weight_mode=weight_mode, num_norm_avg=num_norm_avg)
             self.adata.obsm["X_pca"] = g_weighted
             ge = np.array(list(self.adata.var_names[gkeep]))
@@ -1214,14 +1150,14 @@ class SAM(object):
             return dt
 
         else:
-            distance = self.run_args.get("distance", "correlation")
+            distance = self.adata.uns['run_args'].get("distance", "correlation")
             dt = man.TSNE(metric=distance, **kwargs).fit_transform(
                 self.adata.obsm["X_pca"]
             )
             tsne2d = dt
             self.adata.obsm["X_tsne"] = tsne2d
 
-    def run_umap(self, X="X_pca", metric=None,seed = 0, **kwargs):
+    def run_umap(self, X="X_pca", metric=None,seed = None, **kwargs):
         """Wrapper for umap-learn.
 
         See https://github.com/lmcinnes/umap sklearn for the documentation
@@ -1231,7 +1167,10 @@ class SAM(object):
         import umap as umap
 
         if metric is None:
-            metric = self.run_args.get("distance", "correlation")
+            metric = self.adata.uns['run_args'].get("distance", "correlation")
+            
+        if seed is None:
+            seed = self.adata.uns['run_args'].get('seed',0)
 
         if type(X) is str:
             if X == "":
@@ -1256,8 +1195,8 @@ class SAM(object):
         """
         import scanpy.api as sc
 
-        k = self.run_args.get("k", 20)
-        distance = self.run_args.get("distance", "correlation")
+        k = self.adata.uns['run_args'].get("k", 20)
+        distance = self.adata.uns['run_args'].get("distance", "correlation")
         sc.pp.neighbors(
             self.adata, use_rep=use_rep, n_neighbors=k, metric=distance, method=method
         )
@@ -1283,8 +1222,8 @@ class SAM(object):
     ):
         import scanpy.api as sc
 
-        k = self.run_args.get("k", 20)
-        distance = self.run_args.get("distance", "correlation")
+        k = self.adata.uns['run_args'].get("k", 20)
+        distance = self.adata.uns['run_args'].get("distance", "correlation")
         sc.pp.neighbors(
             self.adata, use_rep=use_rep, n_neighbors=k, metric=distance, method=method
         )
@@ -1301,7 +1240,7 @@ class SAM(object):
             save = False
 
         cl = DBSCAN(eps=eps, metric=metric, **kwargs).fit_predict(X)
-        k = self.run_args.get("k", 20)
+        k = self.adata.uns['run_args'].get("k", 20)
         idx0 = np.where(cl != -1)[0]
         idx1 = np.where(cl == -1)[0]
         if idx1.size > 0 and idx0.size > 0:
@@ -1473,7 +1412,7 @@ class SAM(object):
         self.adata.obs["kmeans_clusters"] = pd.Categorical(cl)
         return cl, km
 
-    def leiden_clustering(self, X=None, res=1, method="modularity"):
+    def leiden_clustering(self, X=None, res=1, method="modularity", seed = 0):
 
         if X is None:
             X = self.adata.obsp["connectivities"]
@@ -1500,10 +1439,10 @@ class SAM(object):
             pass
 
         if method == "significance":
-            cl = leidenalg.find_partition(g, leidenalg.SignificanceVertexPartition)
+            cl = leidenalg.find_partition(g, leidenalg.SignificanceVertexPartition,seed=seed)
         else:
             cl = leidenalg.find_partition(
-                g, leidenalg.RBConfigurationVertexPartition, resolution_parameter=res
+                g, leidenalg.RBConfigurationVertexPartition, resolution_parameter=res,seed=seed
             )
 
         if save:
@@ -1531,7 +1470,7 @@ class SAM(object):
             save = False
 
         if k is None:
-            k = self.run_args.get("k", 20)
+            k = self.adata.uns['run_args'].get("k", 20)
 
         hdb = hdbscan.HDBSCAN(metric="euclidean", **kwargs)
 
@@ -1687,7 +1626,7 @@ class SAM(object):
 
         if "X_knn_avg" not in list(self.adata.layers.keys()):
             print("Performing kNN-averaging...")
-            self.dispersion_ranking_NN()
+            self.dispersion_ranking_NN(save_avgs=True)
         l = self.adata.layers["X_knn_avg"]
         m = l.mean(0).A.flatten()
         cells = np.array(list(self.adata.obs_names))
@@ -1766,4 +1705,4 @@ class SAM(object):
         else:
             from samalg.gui import SAMGUI
             self.SamGui = SAMGUI(self)
-            return self.SamGui.SamPlot
\ No newline at end of file
+            return self.SamGui.SamPlot
