@@ -1,3 +1,4 @@
+import pandas as pd
 import numpy as np
 import scipy as sp
 import os
@@ -9,7 +10,7 @@ from sklearn.utils import check_array, check_random_state
 from scipy import sparse
 import sklearn.utils.sparsefuncs as sf
 from umap.umap_ import nearest_neighbors
-__version__ = "0.7.6"
+__version__ = "0.7.7"
 
 
 def find_corr_genes(sam, input_gene):
@@ -45,22 +46,6 @@ def find_corr_genes(sam, input_gene):
     pw_corr = generate_correlation_map(D_avg.T.A, D_avg[:, input_gene].T.A)
     return all_gene_names[np.argsort(-pw_corr.flatten())]
 
-
-"""
-import hnswlib
-def nearest_neighbors_hnsw(x,ef=200,M=48,n_neighbors = 100):
-    labels = np.arange(x.shape[0])
-    p = hnswlib.Index(space = 'cosine', dim = x.shape[1])
-    p.init_index(max_elements = x.shape[0], ef_construction = ef, M = M)
-    p.add_items(x, labels)
-    p.set_ef(ef)
-    idx, dist = p.knn_query(x, k = n_neighbors)
-    dist = 1-dist
-    dist[dist<0]=0
-    return idx,dist
-"""
-
-
 def _pca_with_sparse(X, npcs, solver='arpack', mu=None, seed=0):
     random_state = check_random_state(seed)
     np.random.set_state(random_state.get_state())
@@ -362,13 +347,27 @@ def convert_annotations(A):
     return y.astype("int")
 
 
+def nearest_neighbors_hnsw(x,ef=200,M=48,n_neighbors = 100):
+    import hnswlib
+    labels = np.arange(x.shape[0])
+    p = hnswlib.Index(space = 'cosine', dim = x.shape[1])
+    p.init_index(max_elements = x.shape[0], ef_construction = ef, M = M)
+    p.add_items(x, labels)
+    p.set_ef(ef)
+    idx, dist = p.knn_query(x, k = n_neighbors)
+    dist = (2-dist)/2
+    return idx,dist
+
+
 def calc_nnm(g_weighted, k, distance=None):
     if g_weighted.shape[0] > 0:
-        # only uses cosine
-        nnm, dists = nearest_neighbors_wrapper(g_weighted, n_neighbors=k, metric=distance)
+        if distance == 'cosine':
+            nnm, dists = nearest_neighbors_hnsw(g_weighted, n_neighbors=k)
+        else:
+            nnm, dists = nearest_neighbors_wrapper(g_weighted, n_neighbors=k, metric=distance)
         EDM = gen_sparse_knn(nnm, dists)
         EDM = EDM.tocsr()
-    else: #try removing this and only use stochastic implementation
+    else: #try removing this and only use stochastic 
         if sp.sparse.issparse(g_weighted):
             g_weighted = g_weighted.A
         dist = compute_distances(g_weighted, distance)
